@@ -2,12 +2,12 @@
 
 #  ************************** Copyrights and license ***************************
 #
-# This file is part of gcovr 8.4, a parsing and reporting tool for gcov.
-# https://gcovr.com/en/8.4
+# This file is part of gcovr 8.6, a parsing and reporting tool for gcov.
+# https://gcovr.com/en/8.6
 #
 # _____________________________________________________________________________
 #
-# Copyright (c) 2013-2025 the gcovr authors
+# Copyright (c) 2013-2026 the gcovr authors
 # Copyright (c) 2013 Sandia Corporation.
 # Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 # the U.S. Government retains certain rights in this software.
@@ -23,15 +23,16 @@ from __future__ import annotations
 from argparse import _ArgumentGroup, ArgumentParser, ArgumentTypeError, SUPPRESS
 from inspect import isclass
 from locale import getpreferredencoding
-import logging
-from typing import Iterable, Any, Optional, Callable, TextIO
+from typing import Iterable, Any, Callable, TextIO
 from dataclasses import dataclass
 import datetime
 import os
 import re
 
+
 from . import formats
-from .timestamps import parse_timestamp
+from .exceptions import SanityCheckError
+from .logging import LOGGER
 from .options import (
     FilterOption,
     GcovrConfigOption,
@@ -44,8 +45,7 @@ from .options import (
     check_percentage,
     relative_path,
 )
-
-LOGGER = logging.getLogger("gcovr")
+from .timestamps import parse_timestamp
 
 
 def timestamp(value: str) -> datetime.datetime:
@@ -57,7 +57,7 @@ def timestamp(value: str) -> datetime.datetime:
         raise ArgumentTypeError(f"{ex}: {value!r}") from None
 
 
-def source_date_epoch() -> Optional[datetime.datetime]:
+def source_date_epoch() -> datetime.datetime | None:
     """Load time from SOURCE_DATE_EPOCH, if it exists.
     See: <https://reproducible-builds.org/docs/source-date-epoch/>
 
@@ -153,12 +153,12 @@ def argument_parser_setup(
             group.add_argument(opt.name, **kwargs)
 
         else:
-            raise AssertionError("Oops, sanity check failed: Unexpected option.")
+            raise SanityCheckError("Unexpected option.")
 
 
 def parse_config_into_dict(
     config_entry_source: Iterable[ConfigEntry],
-    all_options: Optional[Iterable[GcovrConfigOption]] = None,
+    all_options: Iterable[GcovrConfigOption] | None = None,
 ) -> dict[str, Any]:
     """Parse a config file and save the configuration in a dictionary."""
     cfg_dict = dict[str, Any]()
@@ -189,7 +189,7 @@ def _get_value_from_config_entry(
     cfg_entry: ConfigEntry,
     option: GcovrConfigOption,
 ) -> Any:
-    def get_boolean(silent_error: bool = False) -> Optional[bool]:
+    def get_boolean(silent_error: bool = False) -> bool | None:
         try:
             return cfg_entry.value_as_bool
         except ValueError:
@@ -211,7 +211,7 @@ def _get_value_from_config_entry(
     if use_const is False:
         return option.default
     if use_const is not None:
-        raise AssertionError("Oops, sanity check failed: Unexpected entry type.")
+        raise SanityCheckError("Unexpected entry type.")
 
     # parse the value
     value: object
@@ -285,7 +285,7 @@ def _assign_value_to_dict(
     value: Any,
     option: GcovrConfigOption,
     is_single_value: bool,
-    cfg_entry_key: Optional[str] = None,
+    cfg_entry_key: str | None = None,
 ) -> None:
     if option.action == "append" or option.nargs == "*":
         append_target = namespace.setdefault(option.name, [])
@@ -314,7 +314,7 @@ def _assign_value_to_dict(
 
 def merge_options_and_set_defaults(
     partial_namespaces: list[dict[str, Any]],
-    all_options: Optional[list[GcovrConfigOption]] = None,
+    all_options: list[GcovrConfigOption] | None = None,
 ) -> Options:
     """Merge all options into the namespace and set the default values for unused options."""
     if not partial_namespaces:
@@ -390,6 +390,19 @@ GCOVR_CONFIG_OPTION_GROUPS = [
             "executed."
         ),
     },
+    {
+        "key": "llvm_options",
+        "name": "LLVM Options",
+        "description": (
+            "The 'llvm-profdata' tool turns raw coverage files (.profraw) "
+            "into .profdata files which are then exported by 'llvm-cov' into a JSON string. "
+        ),
+    },
+    {
+        "key": "gcov_llvm_options",
+        "name": "GCOV and LLVM Options",
+        "description": ("Options which are applicable for GCOV and LLVM processing."),
+    },
 ]
 
 
@@ -736,11 +749,74 @@ GCOVR_CONFIG_OPTIONS = [
         type=NonEmptyFilterOption,
         default=[],
     ),
+    GcovrConfigOption(
+        "exclude_directory",
+        [
+            "--exclude-directory",
+            "--gcov-exclude-directory",
+            "--gcov-exclude-directories",
+            "--exclude-directories",
+        ],
+        group="filter_options",
+        help=(
+            "Exclude directories that match this regex "
+            "while searching raw coverage files. "
+            "Can be specified multiple times."
+        ),
+        action="append",
+        type=NonEmptyFilterOption,
+        default=[],
+    ),
+    GcovrConfigOption(
+        "keep_intermediate_files",
+        ["-k", "--keep-intermediate-files", "--keep", "--gcov-keep"],
+        config="keep-intermediate-files",
+        group="gcov_llvm_options",
+        help=(
+            "Keep gcov/profdata files after processing. "
+            "This applies both to files that were generated by gcovr, "
+            "or were supplied via the --gcov-use-existing-files/--llvm-use-existing-files option. "
+        ),
+        action="store_true",
+    ),
+    GcovrConfigOption(
+        "delete_input_files",
+        ["-d", "--delete-input-files", "--delete", "--gcov-delete"],
+        config="delete-input-files",
+        group="gcov_llvm_options",
+        help="Delete gcda/profraw files after processing, used gcno files are never deleted.",
+        action="store_true",
+    ),
+    GcovrConfigOption(
+        "trace_include_filter",
+        ["--trace-include"],
+        group="filter_options",
+        help=(
+            "Log output for files that match this filter. "
+            "The output is logged without activating verbose mode. "
+            "Can be specified multiple times."
+        ),
+        action="append",
+        type=NonEmptyFilterOption,
+        default=[],
+    ),
+    GcovrConfigOption(
+        "trace_exclude_filter",
+        ["--trace-exclude"],
+        group="filter_options",
+        help=(
+            "Do not log very verbose output for files that match this filter. "
+            "Can be specified multiple times."
+        ),
+        action="append",
+        type=NonEmptyFilterOption,
+        default=[],
+    ),
     GcovrConfigOption(
         "merge_mode_functions",
         ["--merge-mode-functions"],
         metavar="MERGE_MODE",
-        group="gcov_options",
+        group="gcov_llvm_options",
         choices=(
             "strict",
             "merge-use-line-0",
@@ -765,26 +841,6 @@ GCOVR_CONFIG_OPTIONS = [
         ),
         action="store_true",
     ),
-    GcovrConfigOption(
-        "exclude_internal_functions",
-        ["--include-internal-functions"],
-        group="gcov_options",
-        help=(
-            "Include function coverage of compiler internal functions "
-            "(starting with '__' or '_GLOBAL__sub_I_')."
-        ),
-        action="store_false",
-    ),
-    GcovrConfigOption(
-        "exclude_unreachable_branches",
-        ["--exclude-unreachable-branches"],
-        group="gcov_options",
-        help=(
-            "Exclude branch coverage from lines without useful source code "
-            "(often, compiler-generated 'dead' code)."
-        ),
-        action="store_true",
-    ),
     GcovrConfigOption(
         "exclude_function_lines",
         ["--exclude-function-lines"],
@@ -792,26 +848,6 @@ GCOVR_CONFIG_OPTIONS = [
         help="Exclude coverage from lines defining a function.",
         action="store_true",
     ),
-    GcovrConfigOption(
-        "exclude_noncode_lines",
-        ["--exclude-noncode-lines"],
-        config="exclude-noncode-lines",
-        group="gcov_options",
-        help="Exclude coverage from lines which seem to be non-code.",
-        action="store_true",
-        const_negate=False,
-    ),
-    GcovrConfigOption(
-        "exclude_throw_branches",
-        ["--exclude-throw-branches"],
-        group="gcov_options",
-        help=(
-            "For branch coverage, exclude branches "
-            "that the compiler generates for exception handling. "
-            "This often leads to more 'sensible' coverage reports."
-        ),
-        action="store_true",
-    ),
     GcovrConfigOption(
         "exclude_function",
         ["--exclude-function"],
@@ -857,6 +893,46 @@ GCOVR_CONFIG_OPTIONS = [
         help="Print a warning if a line excluded by comments has a hit counter != 0.",
         action="store_true",
     ),
+    GcovrConfigOption(
+        "exclude_internal_functions",
+        ["--include-internal-functions"],
+        group="gcov_options",
+        help=(
+            "Include function coverage of compiler internal functions "
+            "(starting with '__' or '_GLOBAL__sub_I_')."
+        ),
+        action="store_false",
+    ),
+    GcovrConfigOption(
+        "exclude_unreachable_branches",
+        ["--exclude-unreachable-branches"],
+        group="gcov_options",
+        help=(
+            "Remove branch coverage from lines without useful source code "
+            "(often, compiler-generated 'dead' code)."
+        ),
+        action="store_true",
+    ),
+    GcovrConfigOption(
+        "exclude_noncode_lines",
+        ["--exclude-noncode-lines"],
+        config="exclude-noncode-lines",
+        group="gcov_options",
+        help="Remove coverage from lines which seem to be non-code.",
+        action="store_true",
+        const_negate=False,
+    ),
+    GcovrConfigOption(
+        "exclude_throw_branches",
+        ["--exclude-throw-branches"],
+        group="gcov_options",
+        help=(
+            "For branch coverage, remove branches "
+            "that the compiler generates for exception handling. "
+            "This often leads to more 'sensible' coverage reports."
+        ),
+        action="store_true",
+    ),
     GcovrConfigOption(
         "search_paths",
         config="search-path",
@@ -996,10 +1072,10 @@ class ConfigEntry:
     value: str
     """The un-parsed value."""
 
-    filename: Optional[str] = None
+    filename: str | None = None
     """Path of the config file, for error messages."""
 
-    lineno: Optional[int] = None
+    lineno: int | None = None
     """Line of the entry in the config file, for error messages."""
 
     def __str__(self) -> str:
