@@ -2,12 +2,12 @@
 
 #  ************************** Copyrights and license ***************************
 #
-# This file is part of gcovr 8.4, a parsing and reporting tool for gcov.
-# https://gcovr.com/en/8.4
+# This file is part of gcovr 8.6, a parsing and reporting tool for gcov.
+# https://gcovr.com/en/8.6
 #
 # _____________________________________________________________________________
 #
-# Copyright (c) 2013-2025 the gcovr authors
+# Copyright (c) 2013-2026 the gcovr authors
 # Copyright (c) 2013 Sandia Corporation.
 # Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 # the U.S. Government retains certain rights in this software.
@@ -28,11 +28,13 @@ via ``apply_all_exclusions()``, which is configured via the usual options object
 
 from dataclasses import dataclass, field
 import re
-from typing import Optional
-import logging
 
+from ..data_model.coverage import FileCoverage
+from ..logging import LOGGER
 from ..options import Options
 
+from .markers import ExclusionPredicate, FunctionListByLine, apply_exclusion_markers
+from .noncode import remove_unreachable_branches, remove_noncode_lines
 from .utils import (
     make_is_in_any_range_inclusive,
     apply_exclusion_ranges,
@@ -41,12 +43,6 @@ from .utils import (
     get_functions_by_line,
 )
 
-from ..data_model.coverage import FileCoverage
-from .markers import ExclusionPredicate, FunctionListByLine, apply_exclusion_markers
-from .noncode import remove_unreachable_branches, remove_noncode_lines
-
-LOGGER = logging.getLogger("gcovr")
-
 
 @dataclass
 class ExclusionOptions:
@@ -96,6 +92,7 @@ def apply_all_exclusions(
     *,
     lines: list[str],
     options: ExclusionOptions,
+    activate_trace_logging: bool = False,
 ) -> None:
     """
     Apply all available exclusion mechanisms, if they are enabled by the options.
@@ -103,8 +100,23 @@ def apply_all_exclusions(
     Modifies the FileCoverage in place.
     """
 
+    if options.exclude_internal_functions:
+        remove_internal_functions(
+            filecov, activate_trace_logging=activate_trace_logging
+        )
+
+    if options.exclude_throw_branches:
+        remove_throw_branches(filecov, activate_trace_logging=activate_trace_logging)
+
+    if options.exclude_unreachable_branches:
+        remove_unreachable_branches(
+            filecov, lines=lines, activate_trace_logging=activate_trace_logging
+        )
+
     if options.exclude_noncode_lines:
-        remove_noncode_lines(filecov, lines=lines)
+        remove_noncode_lines(
+            filecov, lines=lines, activate_trace_logging=activate_trace_logging
+        )
 
     if options.respect_exclusion_markers:
         apply_exclusion_markers(
@@ -114,83 +126,49 @@ def apply_all_exclusions(
             exclude_branches_by_pattern=options.exclude_branches_by_pattern,
             exclude_pattern_prefix=options.exclude_pattern_prefix,
             warn_excluded_lines_with_hits=options.warn_excluded_lines_with_hits,
+            activate_trace_logging=activate_trace_logging,
+        )
+
+    if options.exclude_function_lines:
+        exclude_function_definition_lines(
+            filecov, activate_trace_logging=activate_trace_logging
         )
 
     if options.exclude_function:
-        remove_functions(filecov, options.exclude_function)
-
-    if options.exclude_throw_branches:
-        remove_throw_branches(filecov)
-
-    if options.exclude_unreachable_branches:
-        remove_unreachable_branches(filecov, lines=lines)
-
-    if options.exclude_function_lines:
-        remove_function_definition_lines(filecov)
-
-    if options.exclude_internal_functions:
-        remove_internal_functions(filecov)
+        exclude_functions(
+            filecov,
+            options.exclude_function,
+            activate_trace_logging=activate_trace_logging,
+        )
 
 
-def remove_internal_functions(filecov: FileCoverage) -> None:
-    """Remove compiler-generated functions, e.g. for static initialization."""
-
-    # Get all the functions first because we want to remove some of them which will else result in an error.
-    for functioncov in list(filecov.functioncov()):
-        if _function_can_be_excluded(
-            functioncov.mangled_name, functioncov.demangled_name
-        ):
-            LOGGER.debug(
-                "%s Removing symbol %s detected as compiler generated functions, e.g. for static initialization",
-                functioncov.location,
-                functioncov.name,
-            )
-            filecov.remove_function_coverage(functioncov)
-
-
-def _function_can_be_excluded(*names: Optional[str]) -> bool:
-    """Special names for construction/destruction of static objects will be ignored"""
-    return any(
-        name is not None
-        and (name.startswith("__") or name.startswith("_GLOBAL__sub_I_"))
-        for name in names
-    )
-
-
-def remove_function_definition_lines(filecov: FileCoverage) -> None:
+def exclude_function_definition_lines(
+    filecov: FileCoverage, activate_trace_logging: bool
+) -> None:
     """Remove coverage for lines that contain a function definition."""
     # iterate over a shallow copy
     known_function_lines = set(
         (lineno, functioncov.name)
         for functioncov in filecov.functioncov()
-        for lineno in functioncov.count.keys()
+        for lineno in functioncov.linenos
     )
     for linecov in list(filecov.linecov()):
         if (linecov.lineno, linecov.function_name) in known_function_lines or (
             linecov.lineno,
             linecov.demangled_function_name,
         ) in known_function_lines:
-            LOGGER.debug(
-                "%s Removing line of function definition",
-                linecov.location,
-            )
-            filecov.remove_line_coverage(linecov)
-
-
-def remove_throw_branches(filecov: FileCoverage) -> None:
-    """Remove branches annotated as "throw"."""
-    for linecov in filecov.linecov():
-        # iterate over shallow copy
-        for branchcov in list(linecov.branches()):
-            if branchcov.throw:
-                LOGGER.debug(
-                    "%s Removing unreachable branch detected as exception-only code",
+            if activate_trace_logging:
+                LOGGER.trace(
+                    "%s: Removing line of function definition for function %s",
                     linecov.location,
+                    linecov.function_name,
                 )
-                linecov.remove_branch_coverage(branchcov)
+            linecov.exclude()
 
 
-def remove_functions(filecov: FileCoverage, patterns: list[re.Pattern[str]]) -> None:
+def exclude_functions(
+    filecov: FileCoverage, patterns: list[re.Pattern[str]], activate_trace_logging: bool
+) -> None:
     """Remove matching functions"""
     if filecov.functioncov():
         functions_by_line: FunctionListByLine = get_functions_by_line(filecov)
@@ -221,9 +199,12 @@ def remove_functions(filecov: FileCoverage, patterns: list[re.Pattern[str]]) ->
                                 functions_by_line=functions_by_line,
                             )
                         break
-        LOGGER.debug(
-            f"Exclusion range for functions from CLI in {filecov.filename}: {str(exclude_ranges)}."
-        )
+            if activate_trace_logging:
+                LOGGER.trace(
+                    "Exclusion range for functions from CLI in %s: %s.",
+                    filecov.filename,
+                    str(exclude_ranges),
+                )
         exclusion_predicate: ExclusionPredicate = make_is_in_any_range_inclusive(
             exclude_ranges
         )
@@ -233,3 +214,49 @@ def remove_functions(filecov: FileCoverage, patterns: list[re.Pattern[str]]) ->
             branch_is_excluded=exclusion_predicate,
             warn_excluded_lines_with_hits=False,
         )
+
+
+def remove_internal_functions(
+    filecov: FileCoverage, activate_trace_logging: bool
+) -> None:
+    """Remove compiler-generated functions, e.g. for static initialization."""
+
+    # Get all the functions first because we want to remove some of them which will else result in an error.
+    for functioncov in list(filecov.functioncov()):
+        if _function_can_be_excluded(
+            functioncov.mangled_name, functioncov.demangled_name
+        ):
+            if activate_trace_logging:
+                LOGGER.trace(
+                    "%s: Removing symbol %s detected as compiler generated functions, e.g. for static initialization",
+                    functioncov.location,
+                    functioncov.name,
+                )
+            filecov.remove_function_coverage(functioncov)
+
+
+def _function_can_be_excluded(*names: str | None) -> bool:
+    """Special names for construction/destruction of static objects will be ignored"""
+    return any(
+        name is not None
+        and (name.startswith("__") or name.startswith("_GLOBAL__sub_I_"))
+        for name in names
+    )
+
+
+def remove_throw_branches(filecov: FileCoverage, activate_trace_logging: bool) -> None:
+    """Remove branches annotated as "throw"."""
+    for linecov in filecov.linecov():
+        # iterate over shallow copy
+        branchcov_list = list(linecov.branches())
+        for branchcov in branchcov_list:
+            if branchcov.throw:
+                linecov.remove_branch(branchcov)
+        if activate_trace_logging:
+            removed_items = len(branchcov_list) - len(list(linecov.branches()))
+            if removed_items > 0:
+                LOGGER.trace(
+                    "%s: Removing %d unreachable branch(es) detected as exception-only code",
+                    linecov.location,
+                    removed_items,
+                )
