@@ -2,12 +2,12 @@
 
 #  ************************** Copyrights and license ***************************
 #
-# This file is part of gcovr 8.4, a parsing and reporting tool for gcov.
-# https://gcovr.com/en/8.4
+# This file is part of gcovr 8.6, a parsing and reporting tool for gcov.
+# https://gcovr.com/en/8.6
 #
 # _____________________________________________________________________________
 #
-# Copyright (c) 2013-2025 the gcovr authors
+# Copyright (c) 2013-2026 the gcovr authors
 # Copyright (c) 2013 Sandia Corporation.
 # Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 # the U.S. Government retains certain rights in this software.
@@ -17,8 +17,6 @@
 #
 # ****************************************************************************
 
-from typing import Union
-
 from ..data_model.container import CoverageContainer
 from ..options import GcovrConfigOption, Options
 
@@ -27,7 +25,7 @@ class BaseHandler:
     """Base class for a format handler."""
 
     @classmethod
-    def get_options(cls) -> list[Union[GcovrConfigOption, str]]:
+    def get_options(cls) -> list[GcovrConfigOption | str]:
         """Get the options of the format handler"""
         raise AssertionError("Function 'get_options' not implemented.")
 
@@ -46,6 +44,8 @@ class BaseHandler:
             "starting_dir",
             "include_filter",
             "exclude_filter",
+            "trace_include_filter",
+            "trace_exclude_filter",
         ]
         option_dict = {}
         for name in global_options + [
