@@ -2,12 +2,12 @@
 
 #  ************************** Copyrights and license ***************************
 #
-# This file is part of gcovr 8.4, a parsing and reporting tool for gcov.
-# https://gcovr.com/en/8.4
+# This file is part of gcovr 8.6, a parsing and reporting tool for gcov.
+# https://gcovr.com/en/8.6
 #
 # _____________________________________________________________________________
 #
-# Copyright (c) 2013-2025 the gcovr authors
+# Copyright (c) 2013-2026 the gcovr authors
 # Copyright (c) 2013 Sandia Corporation.
 # Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 # the U.S. Government retains certain rights in this software.
@@ -24,10 +24,11 @@ import os
 import re
 import shutil
 import subprocess  # nosec # Commands are trusted.
-from typing import Any, Optional
+from typing import Any
 
 from ...data_model.container import CoverageContainer
 from ...data_model.coverage import FileCoverage
+from ...exceptions import SanityCheckError
 from ...options import Options
 from ...utils import get_md5_hexdigest, write_json_output
 
@@ -121,9 +122,7 @@ def write_report(
 
     def run_git_cmd(*args: str) -> str:
         if git is None:
-            raise AssertionError(
-                "Sanity check failed. Function must only be executed if git is found."
-            )
+            raise SanityCheckError("Function must only be executed if git is found.")
         process = subprocess.run(  # nosec # We execute git
             [git] + list(args),
             stdout=subprocess.PIPE,
@@ -194,20 +193,26 @@ def _make_source_file(filecov: FileCoverage, options: Options) -> dict[str, Any]
         total_line_count = len(contents.splitlines())
 
     # Initialize coverage array and load with line coverage data
-    coverage = list[Optional[int]]()
-    branches = list[Optional[int]]()
+    coverage = list[int | None]()
+    branches = list[int | None]()
     source_file["coverage"] = coverage
     # source_file['branches'] = []
-    for linecov in filecov.linecov():
+    for linecov_collection in filecov.lines(sort=True):
         # Comment lines are not collected in `covdata`, but must
         # be reported to coveralls (fill missing lines)
-        _extend_with_none(coverage, linecov.lineno - 1)
-
-        coverage.append(linecov.count if linecov.is_reportable else None)
+        _extend_with_none(coverage, linecov_collection.lineno - 1)
+        linecov_collection = linecov_collection.merge_lines()
+        linecov = list(linecov_collection.linecov())[0]
+        if linecov_collection.is_reportable:
+            coverage.append(linecov.count)
+        else:
+            coverage.append(None)
 
         # Record branch information (INCOMPLETE/OMITTED)
         for branchno, branchcov in enumerate(
-            filter(lambda branchcov: not branchcov.is_excluded, linecov.branches())
+            branchcov
+            for branchcov in linecov.branches(sort=True)
+            if branchcov.is_reportable
         ):
             branches.append(linecov.lineno)
             branches.append(branchcov.source_block_id_or_0)
@@ -224,6 +229,6 @@ def _make_source_file(filecov: FileCoverage, options: Options) -> dict[str, Any]
     return source_file
 
 
-def _extend_with_none(target: list[Optional[int]], wanted_len: int) -> None:
+def _extend_with_none(target: list[int | None], wanted_len: int) -> None:
     current_len = len(target)
     target.extend(None for _ in range(current_len, wanted_len))
