@@ -2,12 +2,12 @@
 
 #  ************************** Copyrights and license ***************************
 #
-# This file is part of gcovr 8.4, a parsing and reporting tool for gcov.
-# https://gcovr.com/en/8.4
+# This file is part of gcovr 8.6, a parsing and reporting tool for gcov.
+# https://gcovr.com/en/8.6
 #
 # _____________________________________________________________________________
 #
-# Copyright (c) 2013-2025 the gcovr authors
+# Copyright (c) 2013-2026 the gcovr authors
 # Copyright (c) 2013 Sandia Corporation.
 # Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 # the U.S. Government retains certain rights in this software.
@@ -18,25 +18,27 @@
 # ****************************************************************************
 
 import gzip
-from json import loads as json_loads
-import logging
+from json import loads as json_loads, dumps as json_dumps
 import os
 import re
 import shlex
 import subprocess  # nosec # Commands are trusted.
 from threading import Lock
-from typing import Any, Callable, Optional
+from typing import Any, Callable
 
 from ...data_model.container import CoverageContainer
 from ...data_model.merging import get_merge_mode_from_options
+from ...exceptions import SanityCheckError
 from ...decision_analysis import DecisionParser
 from ...exclusions import (
     apply_all_exclusions,
     get_exclusion_options_from_options,
 )
 from ...filter import Filter, is_file_excluded
+from ...logging import LOGGER
 from ...options import Options
 from ...utils import (
+    PRETTY_JSON_INDENT,
     commonpath,
     fix_case_of_path,
     is_fs_case_insensitive,
@@ -48,17 +50,14 @@ from .parser import (
 )
 from .workers import Workers, locked_directory
 
-LOGGER = logging.getLogger("gcovr")
-
 output_re = re.compile(r"[Cc]reating [`'](.*)'$")
 source_error_re = re.compile(
-    r"(?:[Cc](?:annot|ould not) open (?:source|graph) file|: No such file or directory)"
+    r"(?:[Cc](?:annot|ould not) open (?:source|graph|notes) file|: No such file or directory)"
 )
 output_error_re = re.compile(
     r"(?:[Cc](?:annot|ould not) open output file|Operation not permitted|Permission denied|Read-only file system)"
 )
-unknown_cla_re = re.compile(r"Unknown command line argument")
-version_mismatch_re = re.compile(r":version '[']+', prefer '[']+'")
+version_mismatch_re = re.compile(r":version '[^']+', prefer.*'[^']+'")
 
 
 def read_report(options: Options) -> CoverageContainer:
@@ -67,7 +66,7 @@ def read_report(options: Options) -> CoverageContainer:
 
     find_files = find_datafiles
     process_file = process_datafile
-    if options.gcov_files:
+    if options.gcov_use_existing_files:
         find_files = find_existing_gcov_files
         process_file = process_existing_gcov_file
 
@@ -79,14 +78,14 @@ def read_report(options: Options) -> CoverageContainer:
             options.search_paths.append(options.gcov_objdir)
 
     for search_path in options.search_paths:
-        datafiles.update(find_files(search_path, options.gcov_exclude_directory))
+        datafiles.update(find_files(search_path, options.exclude_directory))
 
     # Get coverage data
     with Workers(
         options.gcov_parallel,
         lambda: {"covdata": CoverageContainer(), "to_erase": set(), "options": options},
     ) as pool:
-        LOGGER.debug(f"Pool started with {pool.size()} threads")
+        LOGGER.debug("Pool started with %d threads", pool.size())
         for filename in sorted(datafiles):
             pool.add(process_file, filename)
         try:
@@ -110,28 +109,28 @@ def read_report(options: Options) -> CoverageContainer:
 
 
 def find_existing_gcov_files(
-    search_path: str, gcov_exclude_directory: list[re.Pattern[str]]
+    search_path: str, exclude_directory: list[re.Pattern[str]]
 ) -> list[str]:
     """Find .gcov and .gcov.json.gz files under the given search path."""
     if os.path.isfile(search_path):
-        LOGGER.debug(f"Using given gcov file {search_path}")
+        LOGGER.debug("Using given gcov file %s", search_path)
         gcov_files = [search_path]
     else:
-        LOGGER.debug(f"Scanning directory {search_path} for gcov files...")
+        LOGGER.debug("Scanning directory %s for gcov files...", search_path)
         gcov_files = list(
             search_file(
                 lambda fname: re.compile(r".*\.gcov(?:\.json\.gz)?$").match(fname)
                 is not None,
                 search_path,
-                gcov_exclude_directory=gcov_exclude_directory,
+                exclude_directory=exclude_directory,
             )
         )
-        LOGGER.debug(f"Found {len(gcov_files)} files (and will process all of them)")
+        LOGGER.debug("Found %d files (and will process all of them)", len(gcov_files))
     return gcov_files
 
 
 def find_datafiles(
-    search_path: str, gcov_exclude_directory: list[re.Pattern[str]]
+    search_path: str, exclude_directory: list[re.Pattern[str]]
 ) -> list[str]:
     """Find .gcda and .gcno files under the given search path.
 
@@ -142,16 +141,18 @@ def find_datafiles(
     """
     if os.path.isfile(search_path):
         LOGGER.debug(
-            f"Using given {os.path.splitext(search_path)[1][1:]} file {search_path}"
+            "Using given %s file %s",
+            os.path.splitext(search_path)[1][1:],
+            search_path,
         )
         files = [search_path]
     else:
-        LOGGER.debug(f"Scanning directory {search_path} for gcda/gcno files...")
+        LOGGER.debug("Scanning directory %s for gcda/gcno files...", search_path)
         files = list(
             search_file(
                 lambda fname: re.compile(r".*\.gc(da|no)$").match(fname) is not None,
                 search_path,
-                gcov_exclude_directory=gcov_exclude_directory,
+                exclude_directory=exclude_directory,
             )
         )
     gcda_files = []
@@ -171,7 +172,9 @@ def find_datafiles(
         if os.path.splitext(filename)[0] not in known_file_stems
     ]
     LOGGER.debug(
-        f"Found {len(files)} files (and will process {len(gcda_files) + len(gcno_files)})"
+        "Found %d files (and will process %d)",
+        len(files),
+        len(gcda_files) + len(gcno_files),
     )
     return gcda_files + gcno_files
 
@@ -180,37 +183,58 @@ def find_datafiles(
 # Process a single gcov datafile
 #
 def process_gcov_json_data(
-    data_fname: str, covdata: CoverageContainer, options: Options
+    data_fname: str,
+    covdata: CoverageContainer,
+    options: Options,
 ) -> None:
     """Process a GCOV JSON output."""
+    activate_trace_logging = not is_file_excluded(
+        "trace", data_fname, options.trace_include_filter, options.trace_exclude_filter
+    )
 
-    with gzip.open(data_fname, "rt", encoding="UTF-8") as fh_in:
+    with gzip.open(data_fname, "rt", encoding="utf-8") as fh_in:
         gcov_json_data = json_loads(fh_in.read())
+        if activate_trace_logging:
+            LOGGER.trace(
+                "Parsing gcov data file %s:\n%s<<EOF",
+                data_fname,
+                json_dumps(gcov_json_data, indent=PRETTY_JSON_INDENT),
+            )
 
-    coverage = json.parse_coverage(
+    merge_options = get_merge_mode_from_options(options)
+    for filecov, source_lines in json.parse_coverage(
         data_fname,
-        gcov_json_data=gcov_json_data,
+        gcov_json_data,
         include_filter=options.include_filter,
         exclude_filter=options.exclude_filter,
         ignore_parse_errors=options.gcov_ignore_parse_errors,
         suspicious_hits_threshold=options.gcov_suspicious_hits_threshold,
         source_encoding=options.source_encoding,
-    )
-
-    merge_options = get_merge_mode_from_options(options)
-    for filecov, source_lines in coverage:
-        LOGGER.debug(f"Apply exclusions for {filecov.filename}")
+        activate_trace_logging=activate_trace_logging,
+    ):
+        activate_trace_logging = not is_file_excluded(
+            "trace",
+            filecov.filename,
+            options.trace_include_filter,
+            options.trace_exclude_filter,
+        )
+        if activate_trace_logging:
+            LOGGER.trace("Apply exclusions for %s", filecov.filename)
         apply_all_exclusions(
             filecov,
             lines=source_lines,
             options=get_exclusion_options_from_options(options),
+            activate_trace_logging=activate_trace_logging,
         )
 
         if options.show_decision:
             decision_parser = DecisionParser(filecov, source_lines)
             decision_parser.parse_all_lines()
 
-        LOGGER.debug(f"Merge coverage data for {filecov.filename}")
+        if activate_trace_logging:
+            LOGGER.trace(
+                "Merge coverage data for %s using %s.", filecov.filename, merge_options
+            )
         covdata.insert_file_coverage(filecov, merge_options)
 
 
@@ -219,22 +243,29 @@ def process_gcov_json_data(
 #
 def process_gcov_text_data(
     data_fname: str,
-    gcda_fname: Optional[str],
+    gcda_fname: str | None,
     covdata: CoverageContainer,
     options: Options,
-    current_dir: Optional[str] = None,
+    current_dir: str | None = None,
 ) -> None:
     """Process a GCOV text output."""
+    activate_trace_logging = not is_file_excluded(
+        "trace", data_fname, options.trace_include_filter, options.trace_exclude_filter
+    )
     with open(
         data_fname, "r", encoding=options.source_encoding, errors="replace"
     ) as fh_in:
-        lines = fh_in.read().splitlines()
+        content = fh_in.read()
+        if activate_trace_logging:
+            LOGGER.trace("Parsing gcov data file %s:\n%s<<EOF", data_fname, content)
+        lines = content.splitlines()
 
     # Find the source file
     metadata = text.parse_metadata(
         data_fname,
         lines,
         suspicious_hits_threshold=options.gcov_suspicious_hits_threshold,
+        activate_trace_logging=activate_trace_logging,
     )
     source = metadata.get("Source")
     if source is None:
@@ -257,10 +288,13 @@ def process_gcov_text_data(
         current_dir=current_dir,
     )
 
-    if is_file_excluded(fname, options.include_filter, options.exclude_filter):
+    if is_file_excluded(
+        "source file", fname, options.include_filter, options.exclude_filter
+    ):
         return
 
-    LOGGER.debug(f"Parsing coverage data for file {fname}")
+    if activate_trace_logging:
+        LOGGER.trace("Parsing coverage data for file %s", fname)
     key = os.path.normpath(fname)
 
     filecov, source_lines = text.parse_coverage(
@@ -269,28 +303,37 @@ def process_gcov_text_data(
         filename=key,
         ignore_parse_errors=options.gcov_ignore_parse_errors,
         suspicious_hits_threshold=options.gcov_suspicious_hits_threshold,
+        activate_trace_logging=activate_trace_logging,
+        use_existing_files=options.gcov_use_existing_files,
     )
 
-    LOGGER.debug(f"Apply exclusions for {fname}")
-    apply_all_exclusions(filecov, lines=source_lines, options=options)  # type: ignore [arg-type]
+    if activate_trace_logging:
+        LOGGER.trace("Apply exclusions for %s", fname)
+    apply_all_exclusions(
+        filecov,
+        lines=source_lines,
+        options=get_exclusion_options_from_options(options),
+        activate_trace_logging=activate_trace_logging,
+    )
 
     if options.show_decision:
         decision_parser = DecisionParser(filecov, source_lines)
         decision_parser.parse_all_lines()
 
     merge_mode = get_merge_mode_from_options(options)
-    LOGGER.debug(f"Merge coverage data for {fname} using {str(merge_mode)}")
+    if activate_trace_logging:
+        LOGGER.trace("Merge coverage data for %s using %s.", fname, merge_mode)
     covdata.insert_file_coverage(filecov, merge_mode)
 
 
 def guess_source_file_name(
     source_from_gcov: str,
     data_fname: str,
-    gcda_fname: Optional[str],
+    gcda_fname: str | None,
     root_dir: str,
     starting_dir: str,
-    obj_dir: Optional[str],
-    current_dir: Optional[str] = None,
+    obj_dir: str | None,
+    current_dir: str | None = None,
 ) -> str:
     """Guess the full source filename."""
     if current_dir is None:
@@ -316,14 +359,21 @@ def guess_source_file_name(
         fname = fix_case_of_path(fname)
 
     LOGGER.debug(
-        f"Finding source file corresponding to a gcov data file\n"
-        f"  gcov_fname   {data_fname}\n"
-        f"  current_dir  {current_dir}\n"
-        f"  root         {root_dir}\n"
-        f"  starting_dir {starting_dir}\n"
-        f"  obj_dir      {obj_dir}\n"
-        f"  gcda_fname   {gcda_fname}\n"
-        f"  --> fname    {fname}"
+        "Finding source file corresponding to a gcov data file\n"
+        "  gcov_fname   %s\n"
+        "  current_dir  %s\n"
+        "  root         %s\n"
+        "  starting_dir %s\n"
+        "  obj_dir      %s\n"
+        "  gcda_fname   %s\n"
+        "  --> fname    %s",
+        data_fname,
+        current_dir,
+        root_dir,
+        starting_dir,
+        obj_dir,
+        gcda_fname,
+        fname,
     )
 
     return fname
@@ -359,7 +409,7 @@ def guess_source_file_name_heuristics(  # pylint: disable=too-many-return-statem
     current_dir: str,
     root_dir: str,
     starting_dir: str,
-    obj_dir: Optional[str],
+    obj_dir: str | None,
 ) -> str:
     """Guess the full source filename with path by a heuristic."""
     # 0. Try using the path to the gcov file
@@ -446,7 +496,11 @@ def process_datafile(
     i.e. the object files are in a sibling directory.
     TODO: So far there is no good way to address this case.
     """
-    LOGGER.debug(f"Processing file: {filename}")
+    activate_trace_logging = not is_file_excluded(
+        "trace", filename, options.trace_include_filter, options.trace_exclude_filter
+    )
+    if activate_trace_logging:
+        LOGGER.trace("Processing file: %s", filename)
 
     abs_filename = os.path.abspath(filename).replace(
         os.path.sep, "/"
@@ -482,21 +536,22 @@ def process_datafile(
             chdir=wd,
         )
 
-        if options.gcov_delete:
+        if options.delete_input_files:
             if not abs_filename.endswith("gcno"):
                 to_erase.add(abs_filename)
 
         if done:
             return
 
-    errors_output = "\n\t".join(errors)
+    # Join the errors with proper indention
+    errors_output = "\n\t".join("\n\t\t".join(e.split("\n")) for e in errors)
     errors_output = (
-        f"GCOV produced the following errors processing {filename}:\n"
+        f"GCOV produced the following errors processing {abs_filename!r}:\n"
         f"\t{errors_output}\n"
-        "\t(GCOVR could not infer a working directory that resolved it.)\n"
+        "GCOVR could not infer a working directory that resolved it.\n"
+        f"{'' if options.verbose else 'Use option --verbose to get extended information. '}"
         "To ignore this error use option --gcov-ignore-errors=no_working_dir_found."
     )
-    LOGGER.error(errors_output)
 
     # Check if error shall be ignored
     if options.gcov_ignore_errors is None or not any(
@@ -557,6 +612,9 @@ class GcovProgram:
         def __exit__(self, *_: Any) -> None:
             self.lock.release()
 
+    class GcovExecutionError(Exception):
+        """Exception for errors in gcov execution."""
+
     def __init__(self, cmd: str) -> None:
         with GcovProgram.LockContext(GcovProgram.__lock):
             if not GcovProgram.__cmd:
@@ -569,6 +627,17 @@ class GcovProgram:
                     f"Gcov command must not be changed, expected '{GcovProgram.__cmd}', got '{cmd}'"
                 )
 
+    @classmethod
+    def reset(cls) -> None:
+        """Reset the cached values (for testing purposes)."""
+        with GcovProgram.LockContext(GcovProgram.__lock):
+            cls.__cmd = ""
+            cls.__cmd_split = list[str]()
+            cls.__default_options = list[str]()
+            cls.__exitcode_to_ignore = list[int]([0])
+            cls.__help_output = ""
+            cls.__version_output = ""
+
     def identify_and_cache_capabilities(self) -> None:
         """Check the capabilities of GCOVR once."""
         with GcovProgram.LockContext(GcovProgram.__lock):
@@ -607,9 +676,8 @@ class GcovProgram:
                     GcovProgram.__default_options.append("--preserve-paths")
                 else:
                     LOGGER.warning(
-                        "Options '--hash-filenames' and '--preserve-paths' are not "
-                        f"supported by '{GcovProgram.__cmd}'. Source files with "
-                        "identical file names may result in incorrect coverage."
+                        "Options '--hash-filenames' and '--preserve-paths' are not supported by '%s'. Source files with identical file names may result in incorrect coverage.",
+                        GcovProgram.__cmd,
                     )
 
                 if not self.__check_gcov_help_content("LLVM"):
@@ -669,7 +737,7 @@ class GcovProgram:
         return GcovProgram.__default_options
 
     def __get_gcov_process(
-        self, args: list[str], **kwargs: Any
+        self, args: list[str], trace: bool = False, **kwargs: Any
     ) -> "subprocess.Popen[str]":
         # NB: Currently, we will only parse English output
         env = kwargs.pop("env") if "env" in kwargs else dict(os.environ)
@@ -679,7 +747,8 @@ class GcovProgram:
         if "cwd" not in kwargs:
             kwargs["cwd"] = "."
         cmd = GcovProgram.__cmd_split + args
-        LOGGER.debug(f"Running gcov: '{' '.join(cmd)}' in '{kwargs['cwd']}'")
+        if trace:
+            LOGGER.trace("Running gcov in %s: %s", kwargs["cwd"], shlex.join(cmd))
 
         return subprocess.Popen(  # nosec # We know that we execute gcov tool
             cmd,
@@ -690,7 +759,13 @@ class GcovProgram:
             **kwargs,
         )
 
-    def run_with_args(self, args: list[str], **kwargs: Any) -> tuple[str, str]:
+    def run_with_args(
+        self,
+        args: list[str],
+        cwd: str,
+        activate_trace_logging: bool = False,
+        **kwargs: Any,
+    ) -> tuple[str, str]:
         """Run the gcov program.
 
         >>> import platform
@@ -700,48 +775,66 @@ class GcovProgram:
         ...     GcovProgram("bash").run_with_args(["-c", "exit 1"])
         Traceback (most recent call last):
         ...
-        RuntimeError: GCOV returncode was 1.
+        GcovProgram.GcovExecutionError: GCOV returncode was 1.
         >>> if platform.system() == "Windows":
         ...     GcovProgram("bash").run_with_args(["-c", "exit 1"])
         ... else:
         ...     print("kill not working on Windows")  # doctest: +SKIP
         Traceback (most recent call last):
         ...
-        RuntimeError: GCOV returncode was 4294967295.
+        GcovProgram.GcovExecutionError: GCOV returncode was 4294967295.
         >>> if platform.system() == "Windows":
         ...     print("kill not working on Windows")  # doctest: +SKIP
         ... else:
         ...     GcovProgram("bash").run_with_args(["-c", "kill $$"])
         Traceback (most recent call last):
         ...
-        RuntimeError: GCOV returncode was -15 (exited by signal).
+        GcovProgram.GcovExecutionError: GCOV returncode was -15 (exited by signal).
         >>> if platform.system() == "Windows":
         ...     GcovProgram("bash").run_with_args(["-c", "kill $$"])
         ... else:
         ...     print("kill not working on Windows")  # doctest: +SKIP
         Traceback (most recent call last):
         ...
-        RuntimeError: GCOV returncode was 15.
+        GcovProgram.GcovExecutionError: GCOV returncode was 15.
         """
-        gcov_process = self.__get_gcov_process(args, **kwargs)
-        out, err = gcov_process.communicate()
-        LOGGER.debug(
-            f"GCOV return code was {gcov_process.returncode}, stdout was:\n{out}<<, stderr was:\n{err}<<"
+        process = self.__get_gcov_process(
+            args, cwd=cwd, trace=activate_trace_logging, **kwargs
         )
-        if gcov_process.returncode < 0:
-            raise RuntimeError(
-                f"GCOV returncode was {gcov_process.returncode} (exited by signal).\n"
-                f"Stdout of gcov was >>{out}<< End of stdout\n"
-                f"Stderr of gcov was >>{err}<< End of stderr"
+        out, err = process.communicate()
+
+        def remove_generated_files() -> None:
+            """Remove the generated files from gcov output."""
+            for line in out.splitlines():
+                found = output_re.search(line.strip())
+                if found is not None:
+                    fname = found.group(1)
+                    if not os.path.isabs(fname):
+                        fname = os.path.join(cwd, fname)
+                    if os.path.exists(fname):
+                        os.remove(fname)
+
+        if (
+            process.returncode < 0
+            or process.returncode not in GcovProgram.__exitcode_to_ignore
+        ):
+            remove_generated_files()
+            raise self.GcovExecutionError(
+                f"GCOV returncode was {process.returncode}{' (exited by signal)' if process.returncode < 0 else ''}.\n"
+                f"STDERR >>{err}<< End of STDERR\n"
+                f"STDOUT >>{out}<< End of STDOUT"
             )
 
-        if gcov_process.returncode not in GcovProgram.__exitcode_to_ignore:
-            raise RuntimeError(
-                f"GCOV returncode was {gcov_process.returncode}.\n"
-                f"Stdout of gcov was >>{out}<< End of stdout\n"
-                f"Stderr of gcov was >>{err}<< End of stderr"
+        if version_mismatch_re.search(err):
+            # gcov tossed errors: throw exception
+            raise self.GcovExecutionError(
+                f"Version mismatch gcc/gcov.\nSTDERR >>{err}<< End of STDERR"
             )
 
+        if activate_trace_logging:
+            LOGGER.trace("STDERR >>%s<< End of STDERR", err)
+            LOGGER.trace("STDOUT >>%s<< End of STDOUT", out)
+
         return (out, err)
 
 
@@ -767,6 +860,9 @@ def run_gcov_and_process_files(
                 if os.path.exists(filepath):
                     os.remove(filepath)
 
+        class GcovMessageOnStderr(Exception):
+            """Exception for errors messages of gcov printed to STDOUT."""
+
         filename = None
         out = None
         err = None
@@ -787,6 +883,7 @@ def run_gcov_and_process_files(
                 object_directory = os.path.relpath(object_directory, chdir)
             except OSError:  # pragma: no cover # nosec
                 pass
+
             out, err = gcov_cmd.run_with_args(
                 [
                     abs_filename,
@@ -795,6 +892,12 @@ def run_gcov_and_process_files(
                     object_directory,
                 ],
                 cwd=chdir,
+                activate_trace_logging=not is_file_excluded(
+                    "trace",
+                    abs_filename,
+                    options.trace_include_filter,
+                    options.trace_exclude_filter,
+                ),
             )
 
             # find the files that gcov created
@@ -807,13 +910,6 @@ def run_gcov_and_process_files(
             # Remove the not used files
             remove_existing_files(list(all_gcov_files - active_gcov_files))
 
-            if unknown_cla_re.search(err):
-                # gcov tossed errors: throw exception
-                raise RuntimeError(f"Error in gcov command line: {err}")
-            if version_mismatch_re.search(err):
-                # gcov tossed errors: throw exception
-                raise RuntimeError(f"Version mismatch in gcc/gcov: {err}")
-
             ignore_source_errors = options.gcov_ignore_errors is not None and any(
                 v in options.gcov_ignore_errors for v in ["all", "source_not_found"]
             )
@@ -822,62 +918,73 @@ def run_gcov_and_process_files(
             )
             if (
                 # GCOV did not find source file and error shall not be ignored
-                (source_error_re.search(err) and not ignore_source_errors)
-                # GCOV can not write output file and error shall not be ignored
-                or (output_error_re.search(err) and not ignore_output_errors)
+                source_error_re.search(err) and not ignore_source_errors
             ):
-                # gcov tossed errors: try the next potential_wd
-                error(f"In directory {chdir}:\n{err}")
-                done = False
-            else:
-                if ignore_output_errors:
-                    active_gcov_files = set(
-                        f for f in active_gcov_files if os.path.exists(f)
-                    )
+                raise GcovMessageOnStderr(
+                    "GCOV could not find source file, this can be ignored with --gcov-ignore-errors=source_not_found."
+                )
+            if (
+                # GCOV can not write output file and error shall not be ignored
+                output_error_re.search(err) and not ignore_output_errors
+            ):
+                raise GcovMessageOnStderr(
+                    "GCOV could not write output file, this can be ignored with --gcov-ignore-errors=output_error."
+                )
 
-                # Process *.gcov files
+            if ignore_output_errors:
+                active_gcov_files = set(
+                    f for f in active_gcov_files if os.path.exists(f)
+                )
+
+            if options.keep_intermediate_files:
+                # Keep the files with unique names
+                basename = os.path.basename(abs_filename)
+                renamed_active_gcov_files = set[str]()
                 for gcov_filename in active_gcov_files:
-                    if not os.path.exists(gcov_filename):  # pragma: no cover
-                        raise AssertionError(
-                            f"Sanity check failed, output file {gcov_filename} doesn't exist but no error from GCOV detected."
-                        )
+                    directory, filename = os.path.split(gcov_filename)
+                    new_name = os.path.join(directory, f"{basename}.{filename}")
+                    renamed_active_gcov_files.add(new_name)
+                    os.replace(gcov_filename, new_name)
+                active_gcov_files = renamed_active_gcov_files
 
-                    if gcov_filename.endswith(".gcov"):
-                        process_gcov_text_data(
-                            gcov_filename, filename, covdata, options, chdir
-                        )
-                    elif gcov_filename.endswith(".gcov.json.gz"):
-                        process_gcov_json_data(gcov_filename, covdata, options)
-                    else:  # pragma: no cover
-                        raise RuntimeError(
-                            f"Unknown gcov output format {gcov_filename}."
-                        )
+            # Process *.gcov files
+            for gcov_filename in active_gcov_files:
+                if not os.path.exists(gcov_filename):  # pragma: no cover
+                    raise SanityCheckError(
+                        f"Output file {gcov_filename} doesn't exist but no error from GCOV detected."
+                    )
+                if gcov_filename.endswith(".gcov"):
+                    process_gcov_text_data(
+                        gcov_filename, filename, covdata, options, chdir
+                    )
+                elif gcov_filename.endswith(".gcov.json.gz"):
+                    process_gcov_json_data(gcov_filename, covdata, options)
+                else:  # pragma: no cover
+                    raise RuntimeError(f"Unknown gcov output format {gcov_filename}.")
 
-                done = True
+            done = True
 
         except RuntimeError as exc:
             # If we got an merge assertion error we must end the processing
             done = False
             error(
-                f"Trouble processing {abs_filename!r} with working directory {chdir!r}.\n"
-                f"Stdout of gcov was >>{out}<< End of stdout\n"
-                f"Stderr of gcov was >>{err}<< End of stderr\n"
-                f"Exception was >>{str(exc)}<< End of stderr\n"
-                f"Current processed gcov file was {filename!r}.\n"
-                "Use option --verbose to get extended information."
+                f"With working directory {chdir!r}.\n"
+                f"{str(exc)}\n"
+                f"STDERR >>{err}<< End of STDERR\n"
+                f"STDOUT >>{out}<< End of STDOUT"
             )
+        except GcovMessageOnStderr as exc:
+            done = False
+            error(
+                f"With working directory {chdir!r}.\n"
+                f"{str(exc)}\n"
+                f"STDERR >>{err}<< End of STDERR"
+            )
+        except GcovProgram.GcovExecutionError as exc:
+            done = False
+            error(f"With working directory {chdir!r}.\n{str(exc)}")
         finally:
-            if options.gcov_keep and done:
-                # Keep the files with unique names
-                basename = os.path.basename(abs_filename)
-                for gcov_filename in active_gcov_files:
-                    if os.path.exists(gcov_filename):
-                        directory, filename = os.path.split(gcov_filename)
-                        os.replace(
-                            gcov_filename,
-                            os.path.join(directory, f"{basename}.{filename}"),
-                        )
-            else:
+            if not (options.keep_intermediate_files and done):
                 # Remove the used files
                 remove_existing_files(list(active_gcov_files))
 
@@ -886,8 +993,8 @@ def run_gcov_and_process_files(
 
 def select_gcov_files_from_stdout(
     out: str,
-    include_filter: list[Filter],
-    exclude_filter: list[Filter],
+    include_filter: tuple[Filter, ...],
+    exclude_filter: tuple[Filter, ...],
     chdir: str,
 ) -> tuple[set[str], set[str]]:
     """Parse the output to get the list of files to use and all files (unfiltered)."""
@@ -903,7 +1010,7 @@ def select_gcov_files_from_stdout(
         full = os.path.join(chdir, fname)
         all_files.add(full)
 
-        if is_file_excluded(fname, include_filter, exclude_filter):
+        if is_file_excluded("gcov file", fname, include_filter, exclude_filter):
             continue
 
         active_files.add(full)
@@ -919,9 +1026,16 @@ def process_existing_gcov_file(
 ) -> None:
     """Process an existing GCOV filename."""
     if is_file_excluded(
-        filename, options.gcov_include_filter, options.gcov_exclude_filter
+        "gcov file", filename, options.gcov_include_filter, options.gcov_exclude_filter
     ):
-        LOGGER.debug(f"Excluding gcov file: {filename}")
+        if not is_file_excluded(
+            "trace",
+            filename,
+            options.trace_include_filter,
+            options.trace_exclude_filter,
+        ):
+            LOGGER.trace("Excluding gcov file: %s", filename)
+        return
 
     if filename.endswith(".gcov"):
         process_gcov_text_data(filename, None, covdata, options)
@@ -930,5 +1044,5 @@ def process_existing_gcov_file(
     else:  # pragma: no cover
         raise RuntimeError(f"Unknown gcov output format {filename}.")
 
-    if not options.gcov_keep:
+    if not options.keep_intermediate_files:
         to_erase.add(filename)
