@@ -2,12 +2,12 @@
 
 #  ************************** Copyrights and license ***************************
 #
-# This file is part of gcovr 8.4, a parsing and reporting tool for gcov.
-# https://gcovr.com/en/8.4
+# This file is part of gcovr 8.6, a parsing and reporting tool for gcov.
+# https://gcovr.com/en/8.6
 #
 # _____________________________________________________________________________
 #
-# Copyright (c) 2013-2025 the gcovr authors
+# Copyright (c) 2013-2026 the gcovr authors
 # Copyright (c) 2013 Sandia Corporation.
 # Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 # the U.S. Government retains certain rights in this software.
@@ -20,10 +20,9 @@
 # cspell:ignore xmlcharrefreplace
 
 import functools
-import logging
 import os
 import re
-from typing import Any, Callable, Iterable, Iterator, Optional, Union
+from typing import Any, Callable, Iterable, Iterator
 
 from jinja2 import (
     BaseLoader,
@@ -44,8 +43,6 @@ from pygments.token import _TokenType, Token
 from pygments.style import Style
 from pygments.styles.default import DefaultStyle
 
-from ...exclusions.markers import _EXCLUDE_FLAG, get_markers_regex
-
 from ...data_model.container import CoverageContainer, CoverageContainerDirectory
 from ...data_model.coverage import (
     DecisionCoverageConditional,
@@ -53,8 +50,12 @@ from ...data_model.coverage import (
     DecisionCoverageUncheckable,
     FileCoverage,
     LineCoverage,
+    CoverageStat,
+    DecisionCoverageStat,
 )
-from ...data_model.coverage import CoverageStat, DecisionCoverageStat
+from ...data_model.coverage_dict import FunctioncovKeyType
+from ...exclusions.markers import _EXCLUDE_FLAG, get_markers_regex
+from ...logging import LOGGER
 from ...options import Options
 from ...utils import (
     GZIP_SUFFIX,
@@ -67,7 +68,6 @@ from ...utils import (
 )
 
 
-LOGGER = logging.getLogger("gcovr")
 PYGMENTS_CSS_MARKER = "/* Comment.Preproc */"
 
 
@@ -87,17 +87,24 @@ def get_theme_color(html_theme: str) -> str:
 def templates(options: Options) -> Environment:
     """Get the Jinja2 environment for the templates."""
     # As default use the package loader
-    loader: BaseLoader = PackageLoader(
-        "gcovr.formats.html",
-        package_path=get_theme_name(options.html_theme),
-    )
-
+    loaders: list[BaseLoader] = []
     # If a directory is given files in the directory have higher precedence.
     if options.html_template_dir is not None:
-        loader = ChoiceLoader([FileSystemLoader(options.html_template_dir), loader])
+        loaders.append(FileSystemLoader(options.html_template_dir))
+
+    loaders += [
+        PackageLoader(
+            "gcovr.formats.html",
+            package_path=get_theme_name(options.html_theme),
+        ),
+        PackageLoader(
+            "gcovr.formats.html",
+            package_path="common",
+        ),
+    ]
 
     return Environment(
-        loader=loader,
+        loader=ChoiceLoader(loaders),
         autoescape=True,
         trim_blocks=True,
         lstrip_blocks=True,
@@ -108,7 +115,7 @@ def templates(options: Options) -> Environment:
 def user_templates() -> Environment:
     """Get the Jinja2 environment for the user templates."""
 
-    def load_user_template(template: str) -> Optional[str]:
+    def load_user_template(template: str) -> str | None:
         contents = None
         try:
             with open(template, "rb") as f:
@@ -143,9 +150,7 @@ class CssRenderer:
     def render(options: Options) -> str:
         """Get the rendered CSS content."""
         template = CssRenderer.__load_css_template(options)
-        return template.render(
-            tab_size=options.html_tab_size, single_page=options.html_single_page
-        )
+        return template.render(tab_size=options.html_tab_size)
 
 
 class NullHighlighting:
@@ -206,7 +211,7 @@ class PygmentsHighlighting:
                 else style,
             )
         except ImportError as e:  # pragma: no cover
-            LOGGER.warning(f"No syntax highlighting available: {str(e)}")
+            LOGGER.warning("No syntax highlighting available: %s", str(e))
 
     def get_css(self) -> str:
         """Get the CSS for the syntax highlighting."""
@@ -234,7 +239,7 @@ class PygmentsHighlighting:
 
 
 @functools.lru_cache(maxsize=1)
-def get_formatter(options: Options) -> Union[PygmentsHighlighting, NullHighlighting]:
+def get_formatter(options: Options) -> PygmentsHighlighting | NullHighlighting:
     """Get the formatter for the selected theme."""
     if options.html_syntax_highlighting:
         highlight_style = (
@@ -250,7 +255,7 @@ def get_formatter(options: Options) -> Union[PygmentsHighlighting, NullHighlight
 
 
 def coverage_to_class(
-    coverage: Optional[float], medium_threshold: float, high_threshold: float
+    coverage: float | None, medium_threshold: float, high_threshold: float
 ) -> str:
     """Get the coverage class depending on the threshold."""
     if coverage is None:
@@ -268,17 +273,27 @@ class RootInfo:
     """Class holding the information used in Jinja2 template."""
 
     def __init__(self, options: Options) -> None:
+        self.sort_by = (
+            "filename"
+            if options.sort_key == "filename"
+            else ("branches" if options.sort_branches else "lines")
+        )
+        self.sort_percent = options.sort_key != "uncovered-number"
+        self.sorted = (
+            "sorted-descending" if options.sort_reverse else "sorted-ascending"
+        )
         self.medium_threshold = options.medium_threshold
         self.high_threshold = options.high_threshold
         self.medium_threshold_line = options.medium_threshold_line
         self.high_threshold_line = options.high_threshold_line
         self.medium_threshold_branch = options.medium_threshold_branch
         self.high_threshold_branch = options.high_threshold_branch
-        self.link_function_list = (options.html_details or options.html_nested) and (
-            options.html_single_page != "static"
-        )
+        self.link_function_list = (
+            options.html_details or options.html_nested
+        ) and not (options.html_single_page and options.html_static_report)
         self.relative_anchors = options.html_relative_anchors
         self.single_page = options.html_single_page
+        self.static_report = options.html_static_report
 
         self.version = get_version_for_report()
         self.head = options.html_title
@@ -291,6 +306,7 @@ class RootInfo:
         self.calls = dict[str, Any]()
         self.functions = dict[str, Any]()
         self.lines = dict[str, Any]()
+        self.navigation = dict[str, tuple[str | None, str | None]]()
 
     def set_directory(self, directory: str) -> None:
         """Set the directory for the report."""
@@ -312,19 +328,19 @@ class RootInfo:
         self.decisions = dict_from_stat(stats.decision, self.coverage_class)
         self.calls = dict_from_stat(stats.call, self.coverage_class)
 
-    def line_coverage_class(self, coverage: Optional[float]) -> str:
+    def line_coverage_class(self, coverage: float | None) -> str:
         """Get the coverage class for the line."""
         return coverage_to_class(
             coverage, self.medium_threshold_line, self.high_threshold_line
         )
 
-    def branch_coverage_class(self, coverage: Optional[float]) -> str:
+    def branch_coverage_class(self, coverage: float | None) -> str:
         """Get the coverage class for the branch."""
         return coverage_to_class(
             coverage, self.medium_threshold_branch, self.high_threshold_branch
         )
 
-    def coverage_class(self, coverage: Optional[float]) -> str:
+    def coverage_class(self, coverage: float | None) -> str:
         """Get the coverage class for all other types."""
         return coverage_to_class(coverage, self.medium_threshold, self.high_threshold)
 
@@ -382,18 +398,36 @@ def write_report(
 
     if PYGMENTS_CSS_MARKER in css_data:
         LOGGER.info(
-            "Skip adding of pygments styles since {PYGMENTS_CSS_MARKER!r} found in user stylesheet"
+            "Skip adding of pygments styles since %r found in user stylesheet",
+            PYGMENTS_CSS_MARKER,
         )
     else:
         css_data += get_formatter(options).get_css()
 
+    if options.html_details or options.html_nested:
+        data["ROOT_FNAME"] = os.path.basename(output_file)
+        (output_prefix, output_suffix) = _get_prefix_and_suffix(output_file)
+        functions_output_file = f"{output_prefix}.functions{output_suffix}"
+        data["FUNCTIONS_FNAME"] = os.path.basename(functions_output_file)
+        if options.html_single_page:
+            # Remove the prefix to get shorter links
+            data["FUNCTIONS_FNAME"] = data["FUNCTIONS_FNAME"].split(".", maxsplit=1)[1]
+
+    javascript_data = (
+        None
+        if options.html_static_report
+        else templates(options).get_template("gcovr.js").render(**data).strip()
+    )
+
     if self_contained:
         data["css"] = css_data
+        if javascript_data is not None:
+            data["javascript"] = javascript_data
     else:
         css_output = os.path.splitext(output_file)[0] + ".css"
-        with open_text_for_writing(css_output) as f:
-            f.write(css_data)
-            f.write("\n")
+        with open_text_for_writing(css_output, encoding="utf-8") as fh_out:
+            fh_out.write(css_data)
+            fh_out.write("\n")
 
         if options.html_relative_anchors:
             css_link = os.path.basename(css_output)
@@ -401,9 +435,23 @@ def write_report(
             css_link = css_output
         data["css_link"] = css_link
 
+        if javascript_data is not None:
+            javascript_output = os.path.splitext(output_file)[0] + ".js"
+            with open_text_for_writing(javascript_output) as fh_out:
+                fh_out.write(javascript_data)
+                fh_out.write("\n")
+
+            if options.html_relative_anchors:
+                javascript_link = os.path.basename(javascript_output)
+            else:  # pragma: no cover  Can't be checked because of the reference compare
+                javascript_link = javascript_output
+            data["javascript_link"] = javascript_link
+
     data["theme"] = get_theme_color(options.html_theme)
 
     root_info.set_coverage(covdata)
+    if root_info.link_function_list and root_info.functions["total"] == 0:
+        root_info.link_function_list = False
 
     # Generate the coverage output (on a per-package basis)
     # source_dirs = set()
@@ -420,30 +468,30 @@ def write_report(
         covdata.populate_directories(sorted_keys, options.root_filter)
 
     cdata_fname = dict[str, str]()
-    cdata_sourcefile = dict[str, Optional[str]]()
-    for f in sorted_keys + [v.dirname for v in covdata.directories]:
-        filtered_fname = options.root_filter.sub("", f)
+    cdata_sourcefile = dict[str, str | None]()
+    for fname in sorted_keys + [v.dirname for v in covdata.directories]:
+        filtered_fname = options.root_filter.sub("", fname)
         if filtered_fname != "":
             files.append(filtered_fname)
-        cdata_fname[f] = force_unix_separator(filtered_fname)
+        cdata_fname[fname] = force_unix_separator(filtered_fname)
         if options.html_details or options.html_nested or options.html_single_page:
-            if os.path.normpath(f) == os.path.normpath(options.root_dir):
-                cdata_sourcefile[f] = (
+            if os.path.normpath(fname) == os.path.normpath(options.root_dir):
+                cdata_sourcefile[fname] = (
                     output_file[: -len(GZIP_SUFFIX)]
                     if output_file.endswith(GZIP_SUFFIX)
                     else output_file
                 )
             else:
-                cdata_sourcefile[f] = _make_short_source_filename(
+                cdata_sourcefile[fname] = _make_short_source_filename(
                     output_file, filtered_fname.rstrip(os.sep)
                 )
-                if options.html_single_page and cdata_sourcefile[f] is not None:
+                if options.html_single_page and cdata_sourcefile[fname] is not None:
                     # Remove the prefix to get shorter links
-                    cdata_sourcefile[f] = str(cdata_sourcefile[f]).split(
+                    cdata_sourcefile[fname] = str(cdata_sourcefile[fname]).split(
                         ".", maxsplit=1
                     )[1]
         else:
-            cdata_sourcefile[f] = None
+            cdata_sourcefile[fname] = None
 
     # Define the common root directory, which may differ from options.root_dir
     # when source files share a common prefix.
@@ -457,15 +505,23 @@ def write_report(
         if directory != "":
             root_directory = str(directory) + os.sep
 
-    root_info.set_directory(force_unix_separator(root_directory))
+    previous_fname: str | None = None
+    previous_link_report: str | None = None
+    for fname in sorted(cdata_sourcefile.keys()):
+        root_info.navigation[fname] = (previous_link_report, None)
+        link_report = cdata_sourcefile[fname]
+        if link_report is not None:
+            if root_info.relative_anchors or root_info.single_page:
+                link_report = os.path.basename(link_report)
+        if previous_fname is not None:
+            root_info.navigation[previous_fname] = (
+                root_info.navigation[previous_fname][0],
+                link_report,
+            )
+        previous_fname = fname
+        previous_link_report = link_report
 
-    if options.html_details or options.html_nested:
-        (output_prefix, output_suffix) = _get_prefix_and_suffix(output_file)
-        functions_output_file = f"{output_prefix}.functions{output_suffix}"
-        data["FUNCTIONS_FNAME"] = os.path.basename(functions_output_file)
-        if options.html_single_page:
-            # Remove the prefix to get shorter links
-            data["FUNCTIONS_FNAME"] = data["FUNCTIONS_FNAME"].split(".", maxsplit=1)[1]
+    root_info.set_directory(force_unix_separator(root_directory))
 
     if options.html_single_page:
         write_single_page(
@@ -479,7 +535,7 @@ def write_report(
             data,
         )
     else:
-        if options.html_nested:
+        if options.html_nested and covdata.directories:
             write_directory_pages(
                 options,
                 root_info,
@@ -526,10 +582,10 @@ def write_root_page(
 ) -> None:
     """Generate the root HTML file that contains the high level report."""
     files = []
-    for f in sorted_keys:
+    for fname in sorted_keys:
         files.append(
             get_coverage_data(
-                root_info, covdata[f], cdata_sourcefile[f], cdata_fname[f]
+                root_info, covdata[fname], cdata_sourcefile[fname], cdata_fname[fname]
             )
         )
 
@@ -556,9 +612,9 @@ def write_source_pages(
     """Write a page for each source file."""
     error_no_files_not_found = 0
     all_functions = {}
-    for fname, cdata in covdata.items():
+    for fname, filecov in sorted(covdata.items()):
         file_data, functions, file_not_found = get_file_data(
-            options, root_info, fname, cdata_fname, cdata_sourcefile, cdata
+            options, root_info, fname, cdata_fname, cdata_sourcefile, filecov
         )
         all_functions.update(functions)
         if file_not_found:
@@ -625,7 +681,8 @@ def write_directory_pages(
             filename = cdata_sourcefile[dircov.dirname]
         else:
             LOGGER.warning(
-                f"There's a subdirectory {dircov.dirname!r} that there's no source files within it"
+                "There's a subdirectory %r that there's no source files within it",
+                dircov.dirname,
             )
 
         if filename:
@@ -649,9 +706,9 @@ def write_single_page(
     error_no_files_not_found = 0
     files = []
     all_functions = {}
-    for filename, filecov in covdata.items():
+    for fname, filecov in sorted(covdata.items()):
         file_data, functions, file_not_found = get_file_data(
-            options, root_info, filename, cdata_fname, cdata_sourcefile, filecov
+            options, root_info, fname, cdata_fname, cdata_sourcefile, filecov
         )
         all_functions.update(functions)
         if file_not_found:
@@ -660,14 +717,14 @@ def write_single_page(
         files.append(file_data)
 
     all_files = []
-    for f in sorted_keys:
+    for fname in sorted_keys:
         all_files.append(
             get_coverage_data(
-                root_info, covdata[f], cdata_sourcefile[f], cdata_fname[f]
+                root_info, covdata[fname], cdata_sourcefile[fname], cdata_fname[fname]
             )
         )
     directories = list[dict[str, Any]]([{"entries": all_files}])
-    if root_info.single_page == "js-enabled":
+    if not root_info.static_report:
         for dircov in covdata.directories:
             directories.append(
                 get_directory_data(
@@ -700,9 +757,10 @@ def write_single_page(
 
 def get_coverage_data(
     root_info: RootInfo,
-    cdata: Union[CoverageContainerDirectory, FileCoverage],
+    cdata: CoverageContainerDirectory | FileCoverage,
     link_report: str,
     cdata_fname: str,
+    relative_path: str = "",
 ) -> dict[str, Any]:
     """Get the coverage data"""
 
@@ -713,17 +771,24 @@ def get_coverage_data(
     medium_threshold_branch = root_info.medium_threshold_branch
     high_threshold_branch = root_info.high_threshold_branch
 
-    def coverage_class(coverage: Optional[float]) -> str:
+    def coverage_class(coverage: float | None) -> str:
         return coverage_to_class(coverage, medium_threshold, high_threshold)
 
-    def line_coverage_class(coverage: Optional[float]) -> str:
+    def line_coverage_class(coverage: float | None) -> str:
         return coverage_to_class(coverage, medium_threshold_line, high_threshold_line)
 
-    def branch_coverage_class(coverage: Optional[float]) -> str:
+    def branch_coverage_class(coverage: float | None) -> str:
         return coverage_to_class(
             coverage, medium_threshold_branch, high_threshold_branch
         )
 
+    def sort_value(coverage_stats: CoverageStat | DecisionCoverageStat) -> str:
+        return str(
+            coverage_stats.percent_or("-")
+            if root_info.sort_percent
+            else coverage_stats.total - coverage_stats.covered
+        )
+
     stats = cdata.stats
 
     is_file_with_lines = isinstance(cdata, FileCoverage) and cdata.has_lines()
@@ -735,6 +800,7 @@ def get_coverage_data(
         "class": line_coverage_class(
             stats.line.percent_or(100.0 if is_file_with_lines else None)
         ),
+        "sort": sort_value(stats.line),
     }
 
     branches = {
@@ -743,6 +809,7 @@ def get_coverage_data(
         "excluded": stats.branch.excluded,
         "coverage": stats.branch.percent_or("-"),
         "class": branch_coverage_class(stats.branch.percent),
+        "sort": sort_value(stats.branch),
     }
 
     conditions = {
@@ -751,6 +818,7 @@ def get_coverage_data(
         "excluded": stats.condition.excluded,
         "coverage": stats.condition.percent_or("-"),
         "class": branch_coverage_class(stats.condition.percent),
+        "sort": sort_value(stats.condition),
     }
 
     decisions = {
@@ -759,6 +827,7 @@ def get_coverage_data(
         "unchecked": stats.decision.uncheckable,
         "coverage": stats.decision.percent_or("-"),
         "class": coverage_class(stats.decision.percent),
+        "sort": sort_value(stats.decision),
     }
 
     functions = {
@@ -767,6 +836,7 @@ def get_coverage_data(
         "excluded": stats.function.excluded,
         "coverage": stats.function.percent_or("-"),
         "class": coverage_class(stats.function.percent),
+        "sort": sort_value(stats.function),
     }
 
     calls = {
@@ -775,10 +845,16 @@ def get_coverage_data(
         "excluded": stats.call.excluded,
         "coverage": stats.call.percent_or("-"),
         "class": coverage_class(stats.call.percent),
+        "sort": sort_value(stats.call),
     }
     display_filename = force_unix_separator(
         os.path.relpath(
-            os.path.realpath(cdata_fname), os.path.realpath(root_info.directory)
+            os.path.realpath(cdata_fname),
+            os.path.realpath(
+                os.path.join(root_info.directory, relative_path)
+                if relative_path
+                else root_info.directory
+            ),
         )
     )
 
@@ -811,8 +887,6 @@ def get_directory_data(
         relative_path = ""
     elif relative_path.startswith(root_info.directory):
         relative_path = relative_path[len(root_info.directory) :]
-    if len(relative_path) and relative_path[-1] == "/":
-        relative_path = relative_path[:-1]
     directory_data = dict[str, Any](
         {
             "dirname": (
@@ -840,6 +914,7 @@ def get_directory_data(
                 covdata_dir[key],
                 cdata_sourcefile[fname],
                 cdata_fname[fname],
+                relative_path,
             )
         )
 
@@ -861,6 +936,7 @@ def get_file_data(
 
     file_data = dict[str, Any](
         {
+            "fname": filename,
             "filename": cdata_fname[filename],
             "html_filename": os.path.basename(cdata_sourcefile[filename]),
             "source_lines": [],
@@ -872,12 +948,10 @@ def get_file_data(
             root_info, cdata, cdata_sourcefile[filename], cdata_fname[filename]
         )
     )
-    functions = dict[tuple[str, str, int], dict[str, Any]]()
+    functions = dict[tuple[FunctioncovKeyType, str, int], dict[str, Any]]()
     # Only use demangled names (containing a brace)
-    for functioncov in sorted(
-        cdata.functioncov(), key=lambda functioncov: functioncov.key
-    ):
-        for lineno in sorted(functioncov.count.keys()):
+    for functioncov in cdata.functioncov(key=lambda functioncov: functioncov.key):
+        for lineno in functioncov.linenos:
             f_data = dict[str, Any]()
             f_data["name"] = functioncov.name
             f_data["filename"] = cdata_fname[filename]
@@ -885,6 +959,7 @@ def get_file_data(
             f_data["line"] = lineno
             f_data["count"] = functioncov.count[lineno]
             f_data["blocks"] = functioncov.blocks[lineno]
+            f_data["excluded"] = functioncov.excluded[lineno]
             function_stats = cdata.filter_for_function(functioncov).stats
             f_data["line_coverage"] = function_stats.line.percent_or(100.0)
             f_data["branch_coverage"] = function_stats.branch.percent_or("-")
@@ -899,18 +974,18 @@ def get_file_data(
                 )
             ] = f_data
 
-    def get_linecovs(lineno: int) -> Optional[list[LineCoverage]]:
+    def get_linecovs(lineno: int) -> list[LineCoverage] | None:
         """Get a list of line coverage objects if available for the line."""
         linecovs = cdata.get_line(lineno)
-        return None if linecovs is None else list(linecovs)
+        return None if linecovs is None else list(linecovs.linecov(sort=True))
 
     with chdir(options.root_dir):
         linecov_collections = list(cdata.lines())
         max_line_from_cdata = (
             linecov_collections[-1].lineno if linecov_collections else 0
         )
+        file_not_found = True
         try:
-            file_not_found = True
             with open(
                 filename,
                 "r",
@@ -931,7 +1006,10 @@ def get_file_data(
                     )
                 if lineno < max_line_from_cdata:
                     LOGGER.warning(
-                        f"File {filename} has {lineno} line(s) but coverage data has {max_line_from_cdata} line(s)."
+                        "File %s has %d line(s) but coverage data has %d line(s).",
+                        filename,
+                        lineno,
+                        max_line_from_cdata,
                     )
         except OSError as e:
             if filename.endswith("<stdin>"):
@@ -939,7 +1017,7 @@ def get_file_data(
                 file_info = "!!! File from stdin !!!"
             else:
                 file_info = f"!!! Can't read file: {e.strerror} !!!"
-                LOGGER.warning(f"Can't read file: {e}")
+                LOGGER.warning("Can't read file: %s", e)
             # Python ranges are exclusive. We want to iterate over all lines, including
             # that last line. Thus, we have to add a +1 to include that line.
             for lineno in range(1, max_line_from_cdata + 1):
@@ -952,13 +1030,24 @@ def get_file_data(
                     )
                 )
 
+    file_data["lines"]["uncovered"] = len(
+        [row for row in file_data["source_lines"] if row["covclass"] == "uncoveredLine"]
+    )
+    file_data["lines"]["partial"] = len(
+        [
+            row
+            for row in file_data["source_lines"]
+            if row["covclass"] == "partialCoveredLine"
+        ]
+    )
+
     return file_data, functions, file_not_found
 
 
 def dict_from_stat(
-    stat: Union[CoverageStat, DecisionCoverageStat],
-    coverage_class: Callable[[Optional[float]], str],
-    default: Optional[float] = None,
+    stat: CoverageStat | DecisionCoverageStat,
+    coverage_class: Callable[[float | None], str],
+    default: float | None = None,
 ) -> dict[str, Any]:
     """Get a dictionary from the stats."""
     coverage_default = "-" if default is None else default
@@ -981,7 +1070,7 @@ def dict_from_stat(
 def source_row(
     lineno: int,
     source: str,
-    linecov_list: Optional[list[LineCoverage]],
+    linecov_list: list[LineCoverage] | None,
     html_block_ids: bool,
 ) -> dict[str, Any]:
     """Get information for a row"""
@@ -989,7 +1078,7 @@ def source_row(
     line_conditions = None
     line_decisions = None
     line_calls = None
-    linecount: Union[str, int] = ""
+    linecount: int | str = ""
     covclass = ""
     if linecov_list:
         if all(linecov.is_excluded for linecov in linecov_list):
@@ -1049,7 +1138,9 @@ def source_row_branch(
     """Get branch information for a row"""
     items = list[dict[str, Any]]()
     for branchcov in [
-        branchcov for branchcov in linecov.branches() if branchcov.is_reportable
+        branchcov
+        for branchcov in linecov.branches(sort=True)
+        if branchcov.is_reportable
     ]:
         items.append(
             {
@@ -1066,7 +1157,7 @@ def source_row_branch(
 
     stats = linecov.branch_coverage()
     return {
-        "function_name": linecov.demangled_function_name or linecov.function_name,
+        "function_name": linecov.report_function_name,
         "taken": stats.covered,
         "total": stats.total_with_excluded,
         "branches": items,
@@ -1082,7 +1173,7 @@ def source_row_condition(
 
     conditioncov_list = list(
         conditioncov
-        for conditioncov in linecov.conditions()
+        for conditioncov in linecov.conditions(sort=True)
         if conditioncov.is_reportable
     )
     for conditioncov in conditioncov_list:
@@ -1105,7 +1196,7 @@ def source_row_condition(
             )
     stats = linecov.condition_coverage()
     return {
-        "function_name": linecov.demangled_function_name or linecov.function_name,
+        "function_name": linecov.report_function_name,
         "count": stats.total_with_excluded,
         "covered": stats.covered,
         "condition": items,
@@ -1155,7 +1246,7 @@ def source_row_decision(
         raise RuntimeError(f"Unknown decision type {linecov.decision!r}")
 
     return {
-        "function_name": linecov.demangled_function_name or linecov.function_name,
+        "function_name": linecov.report_function_name,
         "taken": len([i for i in items if i.get("taken", False)]),
         "uncheckable": len([i for i in items if i["uncheckable"]]),
         "total": len(items),
@@ -1168,7 +1259,7 @@ def source_row_call(linecov: LineCoverage) -> dict[str, Any]:
     items = []
 
     for callno, callcov in enumerate(
-        filter(lambda callcov: callcov.is_reportable, linecov.calls())
+        callcov for callcov in linecov.calls(sort=True) if callcov.is_reportable
     ):
         items.append(
             {
@@ -1180,7 +1271,7 @@ def source_row_call(linecov: LineCoverage) -> dict[str, Any]:
 
     stats = linecov.call_coverage()
     return {
-        "function_name": linecov.demangled_function_name or linecov.function_name,
+        "function_name": linecov.report_function_name,
         "invoked": stats.covered,
         "total": stats.total_with_excluded,
         "calls": items,
