@@ -2,12 +2,12 @@
 
 #  ************************** Copyrights and license ***************************
 #
-# This file is part of gcovr 8.4, a parsing and reporting tool for gcov.
-# https://gcovr.com/en/8.4
+# This file is part of gcovr 8.6, a parsing and reporting tool for gcov.
+# https://gcovr.com/en/8.6
 #
 # _____________________________________________________________________________
 #
-# Copyright (c) 2013-2025 the gcovr authors
+# Copyright (c) 2013-2026 the gcovr authors
 # Copyright (c) 2013 Sandia Corporation.
 # Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 # the U.S. Government retains certain rights in this software.
@@ -17,18 +17,28 @@
 #
 # ****************************************************************************
 
-from typing import Union
-
 from ...data_model.container import CoverageContainer
 from ...formats.base import BaseHandler
-from ...options import GcovrConfigOption, OutputOrDefault
+from ...options import (
+    GcovrConfigOption,
+    GcovrDeprecatedConfigOptionAction,
+    OutputOrDefault,
+)
+
+
+class UseLcovFormatVersion(GcovrDeprecatedConfigOptionAction):
+    """Argparse action to map old option --lcov-format-v1 to new option --lcov-format-version=1.x."""
+
+    option = "--lcov-format-version"
+    config = "lcov_format_version"
+    value = "1.x"
 
 
 class LcovHandler(BaseHandler):
     """Class to handle LCOV format."""
 
     @classmethod
-    def get_options(cls) -> list[Union[GcovrConfigOption, str]]:
+    def get_options(cls) -> list[GcovrConfigOption | str]:
         return [
             GcovrConfigOption(
                 "lcov",
@@ -45,11 +55,21 @@ class LcovHandler(BaseHandler):
                 const=OutputOrDefault(None),
             ),
             GcovrConfigOption(
-                "lcov_format_v1",
+                "lcov_format_version",
+                ["--lcov-format-version"],
+                config="lcov_format_version",
+                group="output_options",
+                help="The format version to write.",
+                choices=("1.x", "2.0"),
+                default="2.0",
+            ),
+            GcovrConfigOption(
+                "lcov_format_version",
                 ["--lcov-format-1.x"],
                 group="output_options",
-                help="Write format from LCOV version 1.x instead of 2.x.",
-                action="store_true",
+                help="Deprecated, please use --lcov-format-version=1.x instead.",
+                nargs=0,
+                action=UseLcovFormatVersion,
             ),
             GcovrConfigOption(
                 "lcov_comment",
@@ -63,11 +83,23 @@ class LcovHandler(BaseHandler):
                 ["--lcov-test-name"],
                 group="output_options",
                 metavar="NAME",
-                help="The name used for TN in LCOV file. Default is '{default!s}'.",
-                default="GCOVR report",
+                help=(
+                    "The name used for TN in LCOV file, must not contain spaces. "
+                    "Default is '{default!s}'."
+                ),
+                default="GCOVR_report",
             ),
         ]
 
+    def validate_options(self) -> None:
+        if (
+            self.options.lcov_test_name is not None
+            and " " in self.options.lcov_test_name
+        ):
+            raise RuntimeError(
+                f"The LCOV test name must not contain spaces, got {self.options.lcov_test_name!r}."
+            )
+
     def write_report(self, covdata: CoverageContainer, output_file: str) -> None:
         from .write import write_report  # pylint: disable=import-outside-toplevel # Lazy loading is intended here
 
