@@ -2,12 +2,12 @@
 
 #  ************************** Copyrights and license ***************************
 #
-# This file is part of gcovr 8.4, a parsing and reporting tool for gcov.
-# https://gcovr.com/en/8.4
+# This file is part of gcovr 8.6, a parsing and reporting tool for gcov.
+# https://gcovr.com/en/8.6
 #
 # _____________________________________________________________________________
 #
-# Copyright (c) 2013-2025 the gcovr authors
+# Copyright (c) 2013-2026 the gcovr authors
 # Copyright (c) 2013 Sandia Corporation.
 # Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 # the U.S. Government retains certain rights in this software.
@@ -17,9 +17,6 @@
 #
 # ****************************************************************************
 
-import logging
-from typing import Union
-
 from ...data_model.container import CoverageContainer
 from ...formats.base import BaseHandler
 from ...options import (
@@ -28,8 +25,6 @@ from ...options import (
     OutputOrDefault,
 )
 
-LOGGER = logging.getLogger("gcovr")
-
 
 class UseBranchMetricAction(GcovrDeprecatedConfigOptionAction):
     """Argparse action for mapping deprecated option to new option."""
@@ -43,7 +38,7 @@ class TxtHandler(BaseHandler):
     """Class to handle text format."""
 
     @classmethod
-    def get_options(cls) -> list[Union[GcovrConfigOption, str]]:
+    def get_options(cls) -> list[GcovrConfigOption | str]:
         return [
             # Global options needed for report
             "show_calls",
