@@ -2,12 +2,12 @@
 
 #  ************************** Copyrights and license ***************************
 #
-# This file is part of gcovr 8.4, a parsing and reporting tool for gcov.
-# https://gcovr.com/en/8.4
+# This file is part of gcovr 8.6, a parsing and reporting tool for gcov.
+# https://gcovr.com/en/8.6
 #
 # _____________________________________________________________________________
 #
-# Copyright (c) 2013-2025 the gcovr authors
+# Copyright (c) 2013-2026 the gcovr authors
 # Copyright (c) 2013 Sandia Corporation.
 # Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 # the U.S. Government retains certain rights in this software.
@@ -20,8 +20,8 @@
 import gzip
 from hashlib import md5
 import json
-from typing import Any, Callable, Iterator, Optional
-import logging
+import lzma
+from typing import Any, Callable, Iterator
 import os
 import functools
 import re
@@ -29,13 +29,13 @@ import sys
 from contextlib import contextmanager
 from lxml import etree  # nosec # We only write XML files
 
+from .logging import LOGGER
 from .version import __version__
 
-LOGGER = logging.getLogger("gcovr")
-
 REGEX_VERSION_POSTFIX = re.compile(r"(.+?)(?:\.post\d+)?\.dev.+$")
 PRETTY_JSON_INDENT = 4
 GZIP_SUFFIX = ".gz"
+LZMA_SUFFIX = ".xz"
 
 
 class LoopChecker:
@@ -68,7 +68,7 @@ def is_fs_case_insensitive() -> bool:
         and os.path.exists(cwd.lower())
         and os.path.samefile(cwd.upper(), cwd.lower())
     )
-    LOGGER.debug(f"File system is case {'in' if ret else ''}sensitive.")
+    LOGGER.debug("File system is case %s.", "insensitive" if ret else "sensitive")
 
     return ret
 
@@ -94,7 +94,7 @@ def fix_case_of_path(path: str) -> str:
         if len(matched_filename) == 1:
             path = os.path.join(fix_case_of_path(rest), matched_filename[0])
     except FileNotFoundError:
-        LOGGER.warning(f"Can not fix case of path because {rest} not found.")
+        LOGGER.warning("Can not fix case of path because %s not found.", rest)
 
     return path.replace("\\", "/")
 
@@ -110,7 +110,7 @@ def get_version_for_report() -> str:
 def search_file(
     predicate: Callable[[str], bool],
     path: str,
-    gcov_exclude_directory: list[re.Pattern[str]],
+    exclude_directory: list[re.Pattern[str]],
 ) -> Iterator[str]:
     """
     Given a search path, recursively descend to find files that satisfy a
@@ -131,9 +131,7 @@ def search_file(
         dirs[:] = [
             d
             for d in sorted(dirs)
-            if not any(
-                exc.match(os.path.join(root, d)) for exc in gcov_exclude_directory
-            )
+            if not any(exc.match(os.path.join(root, d)) for exc in exclude_directory)
         ]
         root = os.path.abspath(root)
 
@@ -186,20 +184,20 @@ def commonpath(files: list[str]) -> str:
                 break
         prefix_path = os.path.sep.join(common)
 
-    LOGGER.debug(f"Common prefix path is {prefix_path!r}")
+    LOGGER.debug("Common prefix path is %r.", prefix_path)
 
     # make the path relative and add a trailing slash
     if prefix_path:
         prefix_path = os.path.join(
             os.path.relpath(prefix_path, os.path.realpath(os.getcwd())), ""
         )
-        LOGGER.debug(f"Common relative prefix path is {prefix_path!r}")
+        LOGGER.debug("Common relative prefix path is %r.", prefix_path)
     return prefix_path
 
 
 @contextmanager
 def open_text_for_writing(
-    filename: Optional[str], default_filename: Optional[str] = None, **kwargs: Any
+    filename: str | None, default_filename: str | None = None, **kwargs: Any
 ) -> Iterator[Any]:
     """Context manager to open and close a file for text writing.
 
@@ -216,16 +214,27 @@ def open_text_for_writing(
         if filename.endswith(GZIP_SUFFIX):
             with gzip.open(filename, "wt", **kwargs) as fh_out:
                 yield fh_out
+        elif filename.endswith(LZMA_SUFFIX):
+            with lzma.open(filename, "wt", **kwargs) as fh_out:
+                yield fh_out
         else:
             with open(filename, "wt", **kwargs) as fh_out:  # pylint: disable=unspecified-encoding
                 yield fh_out
     else:
-        yield sys.stdout
+        encoding = kwargs.get("encoding", "utf-8").lower()
+        old_encoding = sys.stdout.encoding
+        try:
+            if old_encoding != encoding:
+                sys.stdout.reconfigure(encoding=encoding)  # type: ignore[union-attr]
+            yield sys.stdout
+        finally:
+            if old_encoding != encoding:
+                sys.stdout.reconfigure(encoding=old_encoding)  # type: ignore[union-attr]
 
 
 @contextmanager
 def open_binary_for_writing(
-    filename: Optional[str],
+    filename: str | None,
     default_filename: str,
     **kwargs: Any,
 ) -> Iterator[Any]:
@@ -240,6 +249,9 @@ def open_binary_for_writing(
         if filename.endswith(GZIP_SUFFIX):
             with gzip.open(filename, "wb", **kwargs) as fh_out:
                 yield fh_out
+        elif filename.endswith(LZMA_SUFFIX):
+            with lzma.open(filename, "wb", **kwargs) as fh_out:
+                yield fh_out
         else:
             # files in write binary mode for UTF-8
             with open(filename, "wb", **kwargs) as fh_out:
@@ -252,7 +264,7 @@ def write_json_output(
     json_dict: dict[str, Any],
     *,
     pretty: bool,
-    filename: Optional[str],
+    filename: str | None,
     default_filename: str,
     **kwargs: Any,
 ) -> None:
@@ -264,9 +276,9 @@ def write_json_output(
 def write_xml_output(
     root: Any,
     *,
-    doctype: Optional[str] = None,
+    doctype: str | None = None,
     pretty: bool,
-    filename: Optional[str],
+    filename: str | None,
     default_filename: str,
     **kwargs: Any,
 ) -> None:
@@ -276,7 +288,7 @@ def write_xml_output(
             etree.tostring(
                 root,
                 pretty_print=pretty,
-                encoding="UTF-8",
+                encoding="utf-8",
                 xml_declaration=True,
                 doctype=doctype,  # type: ignore [arg-type]
             )
@@ -302,3 +314,44 @@ def force_unix_separator(path: str) -> str:
 def get_md5_hexdigest(data: bytes) -> str:
     """Get the MD5 digest of the given bytes."""
     return md5(data, usedforsecurity=False).hexdigest()  # nosec # Not used for security
+
+
+def read_source_file(
+    source_encoding: str, filename: str, max_line_number: int
+) -> list[str]:
+    """Read in the source file and fill up lines if needed."""
+    source_lines: list[bytes] = []
+    try:
+        with open(filename, "rb") as fh_in:
+            source_lines = fh_in.read().splitlines()
+        lines = len(source_lines)
+        if lines < max_line_number:
+            LOGGER.warning(
+                "File %s has %d line(s) but coverage data has %d line(s).",
+                filename,
+                lines,
+                max_line_number,
+            )
+            # GCOV itself adds the /*EOF*/ in the text report if there is no data and we used the same.
+            source_lines += [b"/*EOF*/"] * (max_line_number - lines)
+    except OSError as e:
+        if filename.endswith("<stdin>"):
+            message = (
+                f"Got unreadable source file '{filename}', replacing with empty lines."
+            )
+            LOGGER.info(message)
+        else:
+            # The exception contains the source file name,
+            # e.g. [Errno 2] No such file or directory: 'xy.txt'
+            message = f"Can't read file, using empty lines: {e}"
+            LOGGER.warning(message)
+            # If we can't read the file we use as first line the error
+            # and use empty lines for the rest of the lines.
+        source_lines = [b""] * max_line_number
+        source_lines[0] = f"/* {message} */".encode()
+
+    encoded_source_lines = [
+        line.decode(source_encoding, errors="replace") for line in source_lines
+    ]
+
+    return encoded_source_lines
